@@ -1,7 +1,7 @@
-/* DASDTAB.C    (c) Copyright Roger Bowler, 1999-2007                */
+/* DASDTAB.C    (c) Copyright Roger Bowler, 1999-2009                */
 /*              Hercules Supported DASD definitions                  */
 
-// $Id: dasdtab.c,v 1.40 2008/11/02 21:45:07 rbowler Exp $
+// $Id: dasdtab.c 5444 2009-08-09 16:36:42Z hsg001 $
 
 /*-------------------------------------------------------------------*/
 /* This module contains the tables that define the attributes of     */
@@ -14,7 +14,7 @@
 /* (used with permission)                                            */
 /*-------------------------------------------------------------------*/
 
-// $Log: dasdtab.c,v $
+// $Log$
 // Revision 1.40  2008/11/02 21:45:07  rbowler
 // Correction to dasdtab entries for 3375 and 3380
 //
@@ -65,8 +65,8 @@ static CKDDEV ckdtab[] = {
  {"2311",      0x2311,0x00,0x20,0x00,  200,3,10,    0, 3625,   0, 3625,  0,0x0000,-2,20, 61, 537, 512,  0,0,"2841"},
  {"2311-1",    0x2311,0x00,0x20,0x00,  200,3,10,    0, 3625,   0, 3625,  0,0x0000,-2,20, 61, 537, 512,  0,0,"2841"},
 
- {"2314",      0x2314,0x00,0x20,0x00,  200,3,20,    0, 7294,   0, 7294,  0,0x0000,-2,45,101,2137,2048,  0,0,"2841"},
- {"2314-1",    0x2314,0x00,0x20,0x00,  200,3,20,    0, 7294,   0, 7294,  0,0x0000,-2,45,101,2137,2048,  0,0,"2841"},
+ {"2314",      0x2314,0x00,0x20,0x00,  200,3,20,    0, 7294,   0, 7294,  0,0x0000,-2,45,101,2137,2048,  0,0,"2314"},
+ {"2314-1",    0x2314,0x00,0x20,0x00,  200,3,20,    0, 7294,   0, 7294,  0,0x0000,-2,45,101,2137,2048,  0,0,"2314"},
 
  {"3330",      0x3330,0x01,0x20,0x00,  404,7,19,13165,13030, 135,13165,128,0x0000,-1,56,135,   0,   0,  0,0,"3830"},
  {"3330-1",    0x3330,0x01,0x20,0x00,  404,7,19,13165,13030, 135,13165,128,0x0000,-1,56,135,   0,   0,  0,0,"3830"},
@@ -123,6 +123,7 @@ static CKDDEV ckdtab[] = {
 static CKDCU ckdcutab[] = {
 /*                              func/ type                           */
 /* name          type model code feat code features   ciws --------- */
+ {"2314",       0x2314,0x00,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
  {"2835",       0x2835,0x00,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
  {"2841",       0x2841,0x00,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
  {"3830",       0x3830,0x02,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
@@ -188,6 +189,33 @@ static FBADEV fbatab[] = {
 } ;
 #define FBADEV_NUM (sizeof(fbatab)/FBADEV_SIZE)
 
+#if defined(FEATURE_VM_BLOCKIO)
+static BLKTAB blktab[] = {
+#if 0
+   /* Remove conditional compilation when CKD devices supported */
+   CKDIOT("2305",0x2305,15,10,5,3),
+   CKDIOT("2311",0x2311,6,3,1,0),
+   CKDIOT("2314",0x2314,11,6,3,1),
+   CKDIOT("3330",0x3330,20,11,6,3),
+   CKDIOT("3340",0x3340,12,7,3,2),
+   CKDIOT("3350",0x3350,27,15,8,4),
+   CKDIOT("3375",0x3375,40,25,14,8),
+   CKDIOT("3380",0x3380,35,23,14,7),
+   CKDIOT("3390",0x3390,49,33,21,12),
+   CKDIOT("9345",0x9345,41,28,17,9),
+#endif
+   FBAIOT("0671",0x0671),
+   FBAIOT("3310",0x3310),
+   FBAIOT("3370",0x3370),
+   FBAIOT("9332",0x9332),
+   FBAIOT("9335",0x9335),
+   FBAIOT("9336",0x9336)
+};
+#define BLKTAB_NUM (sizeof(blktab)/BLKTAB_SIZE)
+
+#endif /* defined(FEATURE_VM_BLOCKIO) */
+
+
 /*-------------------------------------------------------------------*/
 /* Lookup a table entry either by name or type                       */
 /*-------------------------------------------------------------------*/
@@ -225,7 +253,17 @@ U32 i;                                  /* Loop Index                */
                 return &fbatab[i];
         }
         return NULL;
-
+#if defined(FEATURE_VM_BLOCKIO)
+    case DASD_STDBLK:
+        for (i = 0; i < (int)BLKTAB_NUM; i++)
+        {
+            if ((name && !strcmp(name, blktab[i].name)) ||
+                (U32)devt == (U32)blktab[i].devt || 
+                (U32)devt == (U32)(blktab[i].devt & 0xff))
+                return &blktab[i];
+        }
+        return NULL;
+#endif /* defined(FEATURE_VM_BLOCKIO) */
     default:
         return NULL;
     }
@@ -263,6 +301,20 @@ int len;
     else if (cu->ciw1 != 0) len = 12;
     else len = 7;
 
+    /* If LEGACYSENSEID ENABLE is not set and device is a 2311
+       or 2314, set devid length to zero to force command reject
+       response to 0xE4 Sense ID command */
+    if (1
+        && !sysblk.legacysenseid
+        && (0
+            || 0x2311 == ckd->devt
+            || 0x2314 == ckd->devt
+           )
+        )
+    {
+      len = 0;
+    }    
+
     return len;
 }
 
