@@ -1,7 +1,7 @@
-/* DIAGNOSE.C   (c) Copyright Roger Bowler, 2000-2007                */
+/* DIAGNOSE.C   (c) Copyright Roger Bowler, 2000-2009                */
 /*              ESA/390 Diagnose Functions                           */
 
-// $Id: diagnose.c,v 1.54 2008/12/29 00:00:54 ivan Exp $
+// $Id: diagnose.c 5548 2009-12-15 15:50:44Z rbowler $
 
 /*-------------------------------------------------------------------*/
 /* This module implements miscellaneous diagnose functions           */
@@ -11,53 +11,9 @@
 /* Additional credits:                                               */
 /*      Hercules-specific diagnose calls by Jay Maynard.             */
 /*      Set/reset bad frame indicator call by Jan Jaeger.            */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2007      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2009      */
 /*-------------------------------------------------------------------*/
 
-// $Log: diagnose.c,v $
-// Revision 1.54  2008/12/29 00:00:54  ivan
-// Change semantics for DIAG8CMD configuration statement
-// Disable command redisplay at the console when NOECHO is set
-// Commands typed with a '-' as the first character are not redisplayed
-//
-// Revision 1.53  2008/11/04 05:58:01  fish
-// Put ensure consistent create_thread ATTR usage change back in,
-// and put original diag 0x308 re-IPL fix back in too.
-//
-// Revision 1.52  2008/11/03 15:38:03  rbowler
-// Fix Error starting thread in diagnose 0x308: Invalid argument
-//
-// Revision 1.51  2008/11/03 15:31:53  rbowler
-// Back out consistent create_thread ATTR modification
-//
-// Revision 1.50  2008/10/18 09:31:04  fish
-// Fix diag 0x308 re-IPL
-//
-// Revision 1.49  2008/08/04 22:06:00  rbowler
-// DIAG308 function codes for Program-directed re-IPL
-//
-// Revision 1.48  2007/12/10 23:12:02  gsmith
-// Tweaks to OPTION_MIPS_COUNTING processing
-//
-// Revision 1.47  2007/06/23 00:04:08  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.46  2007/02/13 19:34:40  fish
-// DIAG  X'9C'  (156)
-//
-// Revision 1.45  2007/01/13 07:17:06  bernard
-// backout ccmask
-//
-// Revision 1.44  2007/01/12 15:22:18  bernard
-// ccmask phase 1
-//
-// Revision 1.43  2007/01/02 18:46:16  fish
-// Fix bug in deconfigure_cpu function & tweak power-off diagnose instructions so that they actually work properly now
-//
-// Revision 1.42  2006/12/08 09:43:20  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #if !defined(_HENGINE_DLL_)
@@ -148,7 +104,7 @@ void *stop_cpus_and_ipl(int *ipltype)
   int i;
   char iplcmd[256];
   int cpustates;
-  int mask;
+  CPU_BITMAP mask;
 
   panel_command("stopall");
   logmsg("HHCDN001I Diagnose 0x308 called: System is re-ipled\n");
@@ -217,7 +173,7 @@ U32   code;
         /* If diag8cmd is not enabled then we are not allowed
          * to manipulate the real machine i.e. hercules itself
          */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
         /* The poweroff diagnose is only valid on the 9221 */
@@ -312,7 +268,7 @@ U32   code;
         /* If diag8cmd is not enabled then we are not allowed
          * to manipulate the real machine i.e. hercules itself
          */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
         /* Process CP command and set condition code */
@@ -392,6 +348,13 @@ U32   code;
         regs->psw.cc = 0;
         break;
 
+    case 0x210:
+    /*---------------------------------------------------------------*/
+    /* Diagnose 210: Retrieve Device Information                     */
+    /*---------------------------------------------------------------*/
+        regs->psw.cc = ARCH_DEP(device_info) (r1, r2, regs);
+        break;
+
     case 0x214:
     /*---------------------------------------------------------------*/
     /* Diagnose 214: Pending Page Release                            */
@@ -431,12 +394,31 @@ U32   code;
         /* This function is implemented as a no-operation */
         regs->GR_L(r2) = 0;
         break;
+        
+        
+#if defined(FEATURE_VM_BLOCKIO)
+    case 0x250:
+    /*---------------------------------------------------------------*/
+    /* Diagnose 250: Standardized Block I/O                          */
+    /*---------------------------------------------------------------*/
+        regs->psw.cc = ARCH_DEP(vm_blockio) (r1, r2, regs);
+        break;
+#endif /*defined(FEATURE_VM_BLOCKIO)*/
 
+
+    case 0x260:
+    /*---------------------------------------------------------------*/
+    /* Diagnose 260: Access Certain Virtual Machine Information      */
+    /*---------------------------------------------------------------*/
+        ARCH_DEP(vm_info) (r1, r2, regs);
+        break;
+        
     case 0x264:
     /*---------------------------------------------------------------*/
     /* Diagnose 264: CP Communication                                */
     /*---------------------------------------------------------------*/
         /* This function is implemented as a no-operation */
+        PTT(PTT_CL_ERR,"*DIAG264",regs->GR_L(r1),regs->GR_L(r2),regs->psw.IA_L);
         regs->psw.cc = 0;
         break;
 
@@ -452,6 +434,7 @@ U32   code;
     /* Diagnose 274: Set Timezone Interrupt Flag                     */
     /*---------------------------------------------------------------*/
         /* This function is implemented as a no-operation */
+        PTT(PTT_CL_ERR,"*DIAG274",regs->GR_L(r1),regs->GR_L(r2),regs->psw.IA_L);
         regs->psw.cc = 0;
         break;
 #endif /*FEATURE_EMULATE_VM*/
@@ -464,7 +447,7 @@ U32   code;
         /* If diag8cmd is not enabled then we are not allowed
          * to manipulate the real machine i.e. hercules itself
          */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
         sysblk.inststep = 0;
@@ -478,7 +461,7 @@ U32   code;
         /* If diag8cmd is not enabled then we are not allowed
          * to manipulate the real machine i.e. hercules itself
          */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
         sysblk.inststep = 1;
@@ -499,7 +482,7 @@ U32   code;
         /* If diag8cmd is not enabled then we are not allowed
          * to manipulate the real machine i.e. hercules itself
          */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
         /* Load 4K block address from R2 register */
@@ -528,7 +511,7 @@ U32   code;
         /* If diag8cmd is not enabled then we are not allowed
          * to manipulate the real machine i.e. hercules itself
          */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
         regs->cpustate = CPUSTATE_STOPPING;
@@ -655,7 +638,7 @@ char *ipltype;                          /* "ipl" or "iplc"           */
             /* If diag8cmd is not enabled then we are not allowed
              * to manipulate the real machine i.e. hercules itself
              */
-        if(!(sysblk.diag8cmd & 0x01))
+        if(!(sysblk.diag8cmd & DIAG8CMD_ENABLE))
             ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
 
             regs->cpustate = CPUSTATE_STOPPING;
