@@ -1,50 +1,13 @@
 /* FEAT390.H    (c) Copyright Jan Jaeger, 2000-2009                  */
 /*              ESA/390 feature definitions                          */
 
-// $Id: feat390.h,v 1.47 2009/01/03 10:58:58 jj Exp $
-//
-// $Log: feat390.h,v $
-// Revision 1.47  2009/01/03 10:58:58  jj
-// Fix storage reference
-// Update path length to 1024
-// Enable SCEDIO in ESA/390 mode
-//
-// Revision 1.46  2009/01/02 19:21:50  jj
-// DVD-RAM IPL
-// RAMSAVE
-// SYSG Integrated 3270 console fixes
-//
-// Revision 1.45  2008/12/27 23:34:37  rbowler
-// Integrated 3270 (SYSG) console send command
-//
-// Revision 1.44  2008/12/06 22:42:37  ivan
-// Enable STFLE in ESA/390 Mode.
-//
-// Revision 1.43  2008/02/27 14:14:23  bernard
-// Implemented feature_security_assist_extension_2
-//
-// Revision 1.42  2008/01/21 14:04:27  rbowler
-// Activate ETF3 and ETF2/ETF3-Enhancements in ESA/390 mode
-//
-// Revision 1.41  2007/11/30 15:14:14  rbowler
-// Permit String-Instruction facility to be activated in S/370 mode
-//
-// Revision 1.40  2007/11/18 22:27:07  rbowler
-// Add Message-Security-Assist-Extension-1 facility to ESA/390
-//
-// Revision 1.39  2007/11/16 13:01:49  rbowler
-// Add HFP-multiply-add/subtract facility to ESA/390
-//
-// Revision 1.38  2007/06/23 00:04:09  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.37  2007/04/25 12:10:27  rbowler
-// Move LFAS,SFASR to IEEE-exception-simulation facility
-//
-// Revision 1.36  2006/12/08 09:43:21  jj
-// Add CVS message log
-//
+// $Id: feat390.h 5454 2009-08-14 14:25:06Z rbowler $
 
+/*-------------------------------------------------------------------*/
+/* This file defines the architectural features which are included   */
+/* at compilation time for ESA/390 mode                              */
+/*-------------------------------------------------------------------*/
+ 
 /* This file MUST NOT contain #undef statements */
 #if defined(OPTION_390_MODE)
 #define _ARCH_390_NAME "ESA/390"
@@ -77,6 +40,7 @@
 #define FEATURE_EXTENDED_TRANSLATION_FACILITY_2
 #define FEATURE_EXTENDED_TRANSLATION_FACILITY_3                 /*@ZA*/
 #define FEATURE_EXTERNAL_INTERRUPT_ASSIST
+#define FEATURE_FAST_SYNC_DATA_MOVER
 #define FEATURE_FETCH_PROTECTION_OVERRIDE
 #define FEATURE_FPS_ENHANCEMENT                                 /*DFP*/
 #define FEATURE_FPS_EXTENSIONS
@@ -97,6 +61,7 @@
 #define FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_1
 #define FEATURE_MESSAGE_SECURITY_ASSIST_EXTENSION_2
 #define FEATURE_MOVE_PAGE_FACILITY_2
+#define FEATURE_MPF_INFO
 #define FEATURE_MSSF_CALL
 #define FEATURE_MULTIPLE_CONTROLLED_DATA_SPACE
 #define FEATURE_MVS_ASSIST
@@ -125,6 +90,7 @@
 #define FEATURE_TRACING
 #define FEATURE_WAITSTATE_ASSIST
 #define FEATURE_STORE_FACILITY_LIST_EXTENDED
+#define FEATURE_VM_BLOCKIO
 // #define FEATURE_VECTOR_FACILITY
 
 #endif /*defined(OPTION_390_MODE)*/
