@@ -1,96 +1,13 @@
 /* FEAT900.H    (c) Copyright Jan Jaeger, 2000-2009                  */
 /*              ESAME feature definitions                            */
 
-// $Id: feat900.h,v 1.89 2009/01/02 19:21:51 jj Exp $
-//
-// $Log: feat900.h,v $
-// Revision 1.89  2009/01/02 19:21:51  jj
-// DVD-RAM IPL
-// RAMSAVE
-// SYSG Integrated 3270 console fixes
-//
-// Revision 1.88  2008/12/29 11:04:29  jj
-// Disable 3270 & ASCII console features
-//
-// Revision 1.87  2008/12/29 11:03:10  jj
-// Move HMC disk I/O functions to scedasd.c
-//
-// Revision 1.86  2008/12/24 22:35:53  rbowler
-// Framework for integrated 3270 and ASCII console features
-//
-// Revision 1.85  2008/12/22 00:29:10  ivan
-// Implement February 2008 z/Arch Compare And Swap And Store Facility 2
-// Update FAQ to reflect change
-// update FAQ to also indicate z/Arch DAT Enhancement is implemented
-//
-// Revision 1.84  2008/08/04 22:06:00  rbowler
-// DIAG308 function codes for Program-directed re-IPL
-//
-// Revision 1.83  2008/04/08 17:13:06  bernard
-// Added execute relative long instruction
-//
-// Revision 1.82  2008/03/23 06:20:47  rbowler
-// Activate Move-with-Optional-Specifications Facility for ESAME
-//
-// Revision 1.81  2008/03/08 23:34:30  rbowler
-// Activate General Instructions Extension Facility for ESAME
-//
-// Revision 1.80  2008/03/01 12:19:04  rbowler
-// Rename new features to include the word facility
-//
-// Revision 1.79  2008/02/29 16:25:48  bernard
-// Enabled Parsing Enhancement Facility
-//
-// Revision 1.78  2008/02/28 11:05:13  rbowler
-// Deactivate new features not yet coded
-//
-// Revision 1.77  2008/02/28 10:11:50  rbowler
-// STFL bit settings for new features in zPOP-06
-//
-// Revision 1.76  2008/02/27 17:09:58  bernard
-// introduce FEATURE_GENERAL_INSTRUCTIONS_EXTENSION_FEATURE
-//
-// Revision 1.75  2008/02/27 14:14:50  bernard
-// Implemented feature_message_security_assist_extension_2
-//
-// Revision 1.74  2007/11/30 15:14:14  rbowler
-// Permit String-Instruction facility to be activated in S/370 mode
-//
-// Revision 1.73  2007/07/19 17:53:55  ivan
-// Disable DIAG 308. Current implementation isn't complete enough to allow
-// for a proper 2.6.21 linux kernel IPL
-//
-// Revision 1.72  2007/06/23 00:04:09  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.71  2007/06/02 13:46:41  rbowler
-// PFPO framework
-//
-// Revision 1.70  2007/05/26 21:45:25  rbowler
-// Activate Compare-and-Swap-and-Store feature
-//
-// Revision 1.69  2007/05/18 21:49:04  rbowler
-// Activate Conditional-SSKE feature
-//
-// Revision 1.68  2007/04/27 10:50:41  rbowler
-// STFL bit 27 for MVCOS
-//
-// Revision 1.67  2007/04/26 21:09:08  rbowler
-// Change SSKE instruction format from RRE to RRF_M
-//
-// Revision 1.66  2007/04/25 15:27:01  rbowler
-// Activate Extract-CPU-Time facility
-//
-// Revision 1.65  2007/04/25 12:10:27  rbowler
-// Move LFAS,SFASR to IEEE-exception-simulation facility
-//
-// Revision 1.64  2007/01/30 16:43:28  rbowler
-// Activate Decimal Floating Point Facility
-//
-// Revision 1.63  2006/12/08 09:43:21  jj
-// Add CVS message log
-//
+// $Id: feat900.h 5569 2009-12-27 14:01:09Z hsg001 $
 
+/*-------------------------------------------------------------------*/
+/* This file defines the architectural features which are included   */
+/* at compilation time for ESAME (z/Architecture) mode               */
+/*-------------------------------------------------------------------*/
+ 
 #if defined(OPTION_900_MODE)
 #define _ARCH_900_NAME "z/Arch" /* also: "ESAME" */
 
@@ -114,9 +31,10 @@
 #define FEATURE_COMPARE_AND_SWAP_AND_STORE_FACILITY_2           /*ISW*/
 #define FEATURE_COMPRESSION
 #define FEATURE_CONDITIONAL_SSKE                                /*407*/
-//#define FEATURE_CONFIGURATION_TOPOLOGY_FACILITY                 /*208*/
+#define FEATURE_CONFIGURATION_TOPOLOGY_FACILITY                 /*208*/
+//#define FEATURE_CPU_MEASUREMENT_COUNTER_FACILITY
+//#define FEATURE_CPU_MEASUREMENT_SAMPLING_FACILITY
 #define FEATURE_CPU_RECONFIG
-#define FEATURE_CPUID_FORMAT_1
 #define FEATURE_DAT_ENHANCEMENT
 #define FEATURE_DAT_ENHANCEMENT_FACILITY_2                      /*@Z9*/
 #define FEATURE_DECIMAL_FLOATING_POINT                          /*DFP*/
@@ -129,6 +47,7 @@
 #define FEATURE_EXECUTE_EXTENSIONS_FACILITY                     /*208*/
 #define FEATURE_EXPANDED_STORAGE
 #define FEATURE_EXPEDITED_SIE_SUBSET
+#define FEATURE_EXTENDED_DIAG204
 #define FEATURE_EXTENDED_IMMEDIATE                              /*@Z9*/
 #define FEATURE_EXTENDED_STORAGE_KEYS
 #define FEATURE_EXTENDED_TOD_CLOCK
@@ -145,7 +64,7 @@
 #define FEATURE_HEXADECIMAL_FLOATING_POINT
 #define FEATURE_HFP_EXTENSIONS
 #define FEATURE_HFP_MULTIPLY_ADD_SUBTRACT
-//#define FEATURE_HFP_UNNORMALIZED_EXTENSION                      /*@Z9*/
+#define FEATURE_HFP_UNNORMALIZED_EXTENSION                      /*@Z9*/
 #define FEATURE_HYPERVISOR
 #define FEATURE_IEEE_EXCEPTION_SIMULATION                       /*407*/
 #define FEATURE_IMMEDIATE_AND_RELATIVE
@@ -164,6 +83,7 @@
 #define FEATURE_MIDAW                                           /*@Z9*/
 #define FEATURE_MOVE_PAGE_FACILITY_2
 #define FEATURE_MOVE_WITH_OPTIONAL_SPECIFICATIONS               /*208*/
+#define FEATURE_MPF_INFO
 #define FEATURE_MVS_ASSIST
 #define FEATURE_PAGE_PROTECTION
 #define FEATURE_PARSING_ENHANCEMENT_FACILITY                    /*208*/
@@ -183,6 +103,7 @@
 #define FEATURE_SENSE_RUNNING_STATUS                            /*@Z9*/
 #define FEATURE_SERVICE_PROCESSOR
 #define FEATURE_SET_ADDRESS_SPACE_CONTROL_FAST
+//#define FEATURE_SET_PROGRAM_PARAMETER_FACILITY
 #define FEATURE_SQUARE_ROOT
 #define FEATURE_STORAGE_KEY_ASSIST
 #define FEATURE_STORAGE_PROTECTION_OVERRIDE
@@ -197,6 +118,7 @@
 #define FEATURE_TOD_CLOCK_STEERING                              /*@Z9*/
 #define FEATURE_TRACING
 #define FEATURE_WAITSTATE_ASSIST
+#define FEATURE_VM_BLOCKIO
 
 #endif /*defined(OPTION_900_MODE)*/
 /* end of FEAT900.H */
