@@ -2,8 +2,8 @@
 // Hercules Interface Configuration Program
 // ====================================================================
 //
-// Copyright    (C) Copyright Roger Bowler, 2000-2007
-//              (C) Copyright James A. Pierson, 2002-2007
+// Copyright    (C) Copyright Roger Bowler, 2000-2009
+//              (C) Copyright James A. Pierson, 2002-2009
 //
 // Based on code originally written by Roger Bowler
 // Modified to communicate via unix sockets.
@@ -19,30 +19,7 @@
 // The exit status is zero if successful, non-zero if error.
 //
 
-// $Id: hercifc.c,v 1.31 2008/06/22 05:54:30 fish Exp $
-//
-// $Log: hercifc.c,v $
-// Revision 1.31  2008/06/22 05:54:30  fish
-// Fix print-formatting issue (mostly in tape modules)
-// that can sometimes, in certain circumstances,
-// cause herc to crash.  (%8.8lx --> I32_FMTX, etc)
-//
-// Revision 1.30  2008/02/19 11:49:19  ivan
-// - Move setting of CPU priority after spwaning timer thread
-// - Added support for Posix 1003.1e capabilities
-//
-// Revision 1.29  2007/08/28 20:14:23  gsmith
-// Fix many TUNSETIFF-EINVAL error messages
-//
-// Revision 1.28  2007/06/23 00:04:10  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.27  2007/03/26 23:02:14  gsmith
-// Suppress spurious error messages from hercifc
-//
-// Revision 1.26  2006/12/08 09:43:25  jj
-// Add CVS message log
-//
+// $Id: hercifc.c 5585 2009-12-30 22:40:19Z rbowler $
 
 #include "hercules.h"
 
@@ -221,7 +198,7 @@ int main( int argc, char **argv )
 
         default:
             snprintf( szMsgBuffer,sizeof(szMsgBuffer),
-                     _("HHCIF004W %s: Unknown request: "I32_FMTX".\n"),
+                     _("HHCIF004W %s: Unknown request: %lX\n"),
                      pszProgName, ctlreq.iCtlOp );
             write( STDERR_FILENO, szMsgBuffer, strlen( szMsgBuffer ) );
             continue;
