@@ -1,30 +1,11 @@
-/* HSTDINC.H    (c) Copyright Roger Bowler, 1999-2007                */
+/* HSTDINC.H    (c) Copyright Roger Bowler, 1999-2009                */
 /*              Hercules precompilation-eligible Header Files        */
 
 /* This file contains #include statements for all of the header      */
 /* files which are not dependent on the mainframe architectural      */
 /* features selected and thus are eligible for precompilation        */
 
-// $Id: hstdinc.h,v 1.16 2008/02/19 11:49:19 ivan Exp $
-//
-// $Log: hstdinc.h,v $
-// Revision 1.16  2008/02/19 11:49:19  ivan
-// - Move setting of CPU priority after spwaning timer thread
-// - Added support for Posix 1003.1e capabilities
-//
-// Revision 1.15  2007/06/23 00:04:11  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.14  2007/01/10 16:30:48  rbowler
-// Include netinet/tcp.h for Unix keepalive settings
-//
-// Revision 1.13  2007/01/10 09:32:39  fish
-// Enable connection keep-alive to try and detect 3270 clients that
-// have died (MSVC only right now; don't know how to do it on *nix)
-//
-// Revision 1.12  2006/12/08 09:43:28  jj
-// Add CVS message log
-//
+// $Id: hstdinc.h 5497 2009-10-27 20:17:03Z fish $
 
 #ifndef _HSTDINC_H
 #define _HSTDINC_H
@@ -80,6 +61,9 @@
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
+#ifndef O_NONBLOCK
+#define O_NONBLOCK 0
+#endif
 #include <limits.h>
 #include <time.h>
 #include <sys/stat.h>
@@ -250,6 +234,12 @@
   #include <unistd.h>           // Unix standard definitions
 #endif
 
+#ifdef C99_FLEXIBLE_ARRAYS
+  #define FLEXIBLE_ARRAY        // ("DEVBLK *memdev[];" syntax is supported)
+#else
+  #define FLEXIBLE_ARRAY 0      // ("DEVBLK *memdev[0];" must be used instead)
+#endif
+
 #include "hostopts.h"           // Must come before htypes.h
 #include "htypes.h"             // Hercules-wide data types
 
