@@ -1,10 +1,10 @@
-/* IO.C         (c) Copyright Roger Bowler, 1994-2007                */
+/* IO.C         (c) Copyright Roger Bowler, 1994-2009                */
 /*              ESA/390 CPU Emulator                                 */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2007      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2007      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2009      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2009      */
 
-// $Id: io.c,v 1.61 2007/06/23 00:04:14 ivan Exp $
+// $Id: io.c 5242 2009-03-05 21:33:26Z jj $
 
 /*-------------------------------------------------------------------*/
 /* This module implements all I/O instructions of the                */
@@ -32,7 +32,7 @@
 /*      64-bit IDAW support - Roger Bowler v209                  @IWZ*/
 /*-------------------------------------------------------------------*/
 
-// $Log: io.c,v $
+// $Log$
 // Revision 1.61  2007/06/23 00:04:14  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -62,6 +62,15 @@
 
 #include "inline.h"
 
+#undef PTIO
+#if defined(FEATURE_CHANNEL_SUBSYSTEM)
+ #define PTIO(_class, _name) \
+ PTT(PTT_CL_ ## _class,_name,regs->GR_L(1),(U32)(effective_addr2 & 0xffffffff),regs->psw.IA_L)
+#else
+ #define PTIO(_class, _name) \
+ PTT(PTT_CL_ ## _class,_name,(U32)(effective_addr2 & 0xffffffff),0,regs->psw.IA_L)
+#endif
+
 #if defined(FEATURE_CHANNEL_SUBSYSTEM)
 
 /*-------------------------------------------------------------------*/
@@ -82,6 +91,8 @@ DEVBLK *dev;                            /* -> device block           */
 #endif
        SIE_INTERCEPT(regs);
 
+    PTIO(IO,"CSCH");
+
     /* Program check if the ssid including lcss is invalid */
     SSID_CHECK(regs);
 
@@ -94,6 +105,7 @@ DEVBLK *dev;                            /* -> device block           */
         || (dev->pmcw.flag5 & PMCW5_V) == 0
         || (dev->pmcw.flag5 & PMCW5_E) == 0)
     {
+        PTIO(ERR,"*CSCH");
 #if defined(_FEATURE_IO_ASSIST)
         SIE_INTERCEPT(regs);
 #endif
@@ -127,6 +139,8 @@ DEVBLK *dev;                            /* -> device block           */
 #endif
        SIE_INTERCEPT(regs);
 
+    PTIO(IO,"HSCH");
+
     /* Program check if the ssid including lcss is invalid */
     SSID_CHECK(regs);
 
@@ -139,6 +153,7 @@ DEVBLK *dev;                            /* -> device block           */
         || (dev->pmcw.flag5 & PMCW5_V) == 0
         || (dev->pmcw.flag5 & PMCW5_E) == 0)
     {
+        PTIO(ERR,"*HSCH");
 #if defined(_FEATURE_IO_ASSIST)
         SIE_INTERCEPT(regs);
 #endif
@@ -168,6 +183,8 @@ PMCW    pmcw;                           /* Path management ctl word  */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"MSCH");
+
     FW_CHECK(effective_addr2, regs);
 
     /* Fetch the updated path management control word */
@@ -195,6 +212,7 @@ PMCW    pmcw;                           /* Path management ctl word  */
     /* Condition code 3 if subchannel does not exist */
     if (dev == NULL)
     {
+        PTIO(ERR,"*MSCH");
         regs->psw.cc = 3;
         return;
     }
@@ -202,6 +220,7 @@ PMCW    pmcw;                           /* Path management ctl word  */
     /* If the subchannel is invalid then return cc0 */
     if (!(dev->pmcw.flag5 & PMCW5_V))
     {
+        PTIO(ERR,"*MSCH");
         regs->psw.cc = 0;
         return;
     }
@@ -218,6 +237,7 @@ PMCW    pmcw;                           /* Path management ctl word  */
     if ((dev->scsw.flag3 & SCSW3_SC_PEND)
       && !(dev->scsw.flag3 & SCSW3_SC_INTER))
     {
+        PTIO(ERR,"*MSCH");
         regs->psw.cc = 1;
         release_lock (&dev->lock);
         return;
@@ -226,6 +246,7 @@ PMCW    pmcw;                           /* Path management ctl word  */
     /* Condition code 2 if subchannel is busy */
     if (dev->busy || IOPENDING(dev))
     {
+        PTIO(ERR,"*MSCH");
         regs->psw.cc = 2;
         release_lock (&dev->lock);
         return;
@@ -301,6 +322,8 @@ BYTE    chpid;
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"RCHP");
+
     /* Program check if reg 1 bits 0-23 not zero */
     if(regs->GR_L(1) & 0xFFFFFF00)
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
@@ -339,6 +362,8 @@ DEVBLK *dev;                            /* -> device block           */
 #endif
        SIE_INTERCEPT(regs);
 
+    PTIO(IO,"RSCH");
+
     /* Program check if the ssid including lcss is invalid */
     SSID_CHECK(regs);
 
@@ -351,6 +376,7 @@ DEVBLK *dev;                            /* -> device block           */
         || (dev->pmcw.flag5 & PMCW5_V) == 0
         || (dev->pmcw.flag5 & PMCW5_E) == 0)
     {
+        PTIO(ERR,"*RSCH");
 #if defined(_FEATURE_IO_ASSIST)
         SIE_INTERCEPT(regs);
 #endif
@@ -379,6 +405,8 @@ VADR    effective_addr2;                /* Effective address         */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"SAL");
+
     if(regs->GR_L(1) & 0x8000FFFF)
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
     else
@@ -403,6 +431,8 @@ VADR    effective_addr2;                /* Effective address         */
 #endif
         SIE_INTERCEPT(regs);
 
+    PTIO(IO,"SCHM");
+
     /* Reserved bits in gpr1 must be zero */
     if (regs->GR_L(1) & CHM_GPR1_RESV)
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
@@ -483,6 +513,8 @@ ORB     orb;                            /* Operation request block   */
 #endif
         SIE_INTERCEPT(regs);
 
+    PTIO(IO,"SSCH");
+
     FW_CHECK(effective_addr2, regs);
 
     /* Fetch the operation request block */
@@ -519,6 +551,7 @@ ORB     orb;                            /* Operation request block   */
         || (dev->pmcw.flag5 & PMCW5_E) == 0
         || (orb.lpm & dev->pmcw.pam) == 0)
     {
+        PTIO(ERR,"*SSCH");
 #if defined(_FEATURE_IO_ASSIST)
         SIE_INTERCEPT(regs);
 #endif
@@ -562,6 +595,8 @@ BYTE    work[32];                       /* Work area                 */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"STCPS");
+
     /* Program check if operand not on 32 byte boundary */
     if ( effective_addr2 & 0x0000001F )
         ARCH_DEP(program_interrupt) (regs, PGM_SPECIFICATION_EXCEPTION);
@@ -586,6 +621,8 @@ U32     n;                              /* Integer work area         */
 
     S(inst, regs, b2, effective_addr2);
 
+    PTIO(IO,"STCRW");
+
     PRIV_CHECK(regs);
 
     SIE_INTERCEPT(regs);
@@ -624,6 +661,8 @@ SCHIB   schib;                          /* Subchannel information blk*/
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"STSCH");
+
     /* Program check if the ssid including lcss is invalid */
     SSID_CHECK(regs);
 
@@ -633,6 +672,7 @@ SCHIB   schib;                          /* Subchannel information blk*/
     /* Set condition code 3 if subchannel does not exist */
     if (dev == NULL)
     {
+        PTIO(ERR,"*STSCH");
         regs->psw.cc = 3;
         return;
     }
@@ -689,6 +729,8 @@ RADR    pfx;                            /* Prefix                    */
 #endif
        SIE_INTERCEPT(regs);
 
+    PTIO(IO,"TPI");
+
     FW_CHECK(effective_addr2, regs);
 
     /* validate operand before taking any action */
@@ -792,6 +834,8 @@ int     cc;                             /* Condition Code            */
 #endif
         SIE_INTERCEPT(regs);
 
+    PTIO(IO,"TSCH");
+
     FW_CHECK(effective_addr2, regs);
 
     /* Program check if the ssid including lcss is invalid */
@@ -806,6 +850,7 @@ int     cc;                             /* Condition Code            */
         || (dev->pmcw.flag5 & PMCW5_V) == 0
         || (dev->pmcw.flag5 & PMCW5_E) == 0)
     {
+        PTIO(ERR,"*TSCH");
 #if defined(_FEATURE_IO_ASSIST)
         SIE_INTERCEPT(regs);
 #endif
@@ -851,6 +896,8 @@ DEVBLK *dev;                            /* -> device block           */
 #endif
        SIE_INTERCEPT(regs);
 
+    PTIO(IO,"XSCH");
+
     /* Program check if the ssid including lcss is invalid */
     SSID_CHECK(regs);
 
@@ -863,6 +910,7 @@ DEVBLK *dev;                            /* -> device block           */
         || (dev->pmcw.flag5 & PMCW5_V) == 0
         || (dev->pmcw.flag5 & PMCW5_E) == 0)
     {
+        PTIO(ERR,"*XSCH");
 #if defined(_FEATURE_IO_ASSIST)
         SIE_INTERCEPT(regs);
 #endif
@@ -896,6 +944,7 @@ VADR    ccwaddr;                        /* CCW address for start I/O */
 BYTE    ccwkey;                         /* Bits 0-3=key, 4=7=zeroes  */
 
     S(inst, regs, b2, effective_addr2);
+
 #if defined(FEATURE_ECPSVM)
     if((inst[1])!=0x02)
     {
@@ -910,10 +959,13 @@ BYTE    ccwkey;                         /* Bits 0-3=key, 4=7=zeroes  */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"SIO");
+
     /* Locate the device block */
     if(regs->chanset == 0xFFFF
       || !(dev = find_device_by_devnum (regs->chanset,effective_addr2)) )
     {
+        PTIO(ERR,"*SIO");
         regs->psw.cc = 3;
         return;
     }
@@ -953,10 +1005,13 @@ DEVBLK *dev;                            /* -> device block for SIO   */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"TIO");
+
     /* Locate the device block */
     if(regs->chanset == 0xFFFF
       || !(dev = find_device_by_devnum (regs->chanset,effective_addr2)) )
     {
+        PTIO(ERR,"*TIO");
         regs->psw.cc = 3;
         return;
     }
@@ -989,10 +1044,13 @@ DEVBLK *dev;                            /* -> device block for SIO   */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"HIO");
+
     /* Locate the device block */
     if(regs->chanset == 0xFFFF
       || !(dev = find_device_by_devnum (regs->chanset,effective_addr2)) )
     {
+        PTIO(ERR,"*HIO");
         regs->psw.cc = 3;
         return;
     }
@@ -1019,6 +1077,8 @@ U16     tch_ctl;
 
     PRIV_CHECK(regs);
 
+    PTIO(IO,"TCH");
+
 #if defined(_FEATURE_SIE)
     if(!SIE_MODE(regs))
     {
@@ -1056,6 +1116,8 @@ VADR    effective_addr2;                /* Effective address         */
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"STIDC");
+
     /* Store Channel ID and set condition code */
     regs->psw.cc =
         stchan_id (regs, effective_addr2 & 0xFF00);
@@ -1079,11 +1141,14 @@ int     i;
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"CONCS");
+
     effective_addr2 &= 0xFFFF;
 
     /* Hercules has as many channelsets as CSS's */
     if(effective_addr2 >= FEATURE_LCSS_MAX)
     {
+        PTIO(ERR,"*CONCS");
         regs->psw.cc = 3;
         return;
     }
@@ -1142,9 +1207,12 @@ int     i;
 
     SIE_INTERCEPT(regs);
 
+    PTIO(IO,"DISCS");
+
     /* Hercules has as many channelsets as CSS's */
     if(effective_addr2 >= FEATURE_LCSS_MAX)
     {
+        PTIO(ERR,"*DISCS");
         regs->psw.cc = 3;
         return;
     }
