@@ -1,4 +1,4 @@
-/* SIE.C        (c) Copyright Jan Jaeger, 1999-2007                  */
+/* SIE.C        (c) Copyright Jan Jaeger, 1999-2009                  */
 /*              Interpretive Execution                               */
 
 /*      This module contains the SIE instruction as                  */
@@ -10,9 +10,9 @@
 
 /* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2007      */
 
-// $Id: sie.c,v 1.109 2008/08/21 18:34:48 fish Exp $
+// $Id: sie.c 5249 2009-03-06 15:33:02Z jj $
 //
-// $Log: sie.c,v $
+// $Log$
 // Revision 1.109  2008/08/21 18:34:48  fish
 // Fix i/o-interrupt-queue race condition
 //
@@ -279,6 +279,8 @@ U64     dreg;
 
     PRIV_CHECK(regs);
 
+    PTT(PTT_CL_SIE,"SIE", regs->GR_L(14), regs->GR_L(15), (U32)(effective_addr2 & 0xffffffff));
+
     SIE_PERFMON(SIE_PERF_ENTER);
 
 #if !defined(FEATURE_ESAME) && !defined(FEATURE_MULTIPLE_CONTROLLED_DATA_SPACE)
@@ -743,6 +745,36 @@ void ARCH_DEP(sie_exit) (REGS *regs, int code)
 {
 int     n;
 
+#if defined(OPTION_PTTRACE)
+    if(pttclass & PTT_CL_SIE)
+    {
+    U32  nt1 = 0, nt2 = 0;
+    BYTE *ip;
+        if(!GUESTREGS->instinvalid)
+        {
+
+            if(GUESTREGS->ip[0] == 0x44
+#if defined(FEATURE_EXECUTE_EXTENSIONS_FACILITY)
+               || (GUESTREGS->ip[0] == 0xc6 && !(GUESTREGS->ip[1] & 0x0f))
+#endif /*defined(FEATURE_EXECUTE_EXTENSIONS_FACILITY)*/
+                                                                           )
+            {
+                ip = GUESTREGS->exinst;
+                nt2 = (GUESTREGS->ip[0] == 0x44) ? 0x44 : ((0xc6 << 8) | (GUESTREGS->ip[1] & 0x0f));
+            }
+            else
+                ip = GUESTREGS->ip;
+            nt1 = (ip[0] << 24) | (ip[1] << 16);
+            if(ILC(ip[0]) > 2)
+               nt1 |= (ip[2] << 8) | ip[3];
+            if(ILC(ip[0]) > 4)
+               nt2 |= (ip[4] << 24) | (ip[5] << 16);
+        }
+
+        PTT(PTT_CL_SIE,"*SIE", nt1, nt2, code);
+    }
+#endif /*defined(OPTION_PTTRACE)*/
+
 #if defined(SIE_DEBUG)
     logmsg(_("SIE: interception code %d\n"),code);
     ARCH_DEP(display_inst) (GUESTREGS,
@@ -1174,6 +1206,8 @@ int     zone;                           /* Zone number               */
 
     SIE_INTERCEPT(regs);
 
+    PTT(PTT_CL_IO,"STZP", regs->GR_L(1), regs->GR_L(2),regs->psw.IA_L);
+
     FW_CHECK(regs->GR(2), regs);
 
     zone = regs->GR_LHLCL(1);
@@ -1215,12 +1249,15 @@ RADR    mso,                            /* Main Storage Origin       */
 
     SIE_INTERCEPT(regs);
 
+    PTT(PTT_CL_IO,"SZP", regs->GR_L(1), regs->GR_L(2),regs->psw.IA_L);
+
     FW_CHECK(regs->GR(2), regs);
 
     zone = regs->GR_LHLCL(1);
 
     if(zone == 0 || zone >= FEATURE_SIE_MAXZONES)
     {
+        PTT(PTT_CL_ERR,"*SZP", regs->GR_L(1), regs->GR_L(2),regs->psw.IA_L);
         regs->psw.cc = 3;
         return;
     }
@@ -1270,6 +1307,8 @@ int     zone;                           /* Zone number               */
 
     SIE_INTERCEPT(regs);
 
+    PTT(PTT_CL_IO,"TPZI", regs->GR_L(1),(U32)(effective_addr2 & 0xffffffff),regs->psw.IA_L);
+
     FW_CHECK(regs->GR(2), regs);
 
     /* Perform serialization and checkpoint-synchronization */
@@ -1280,6 +1319,7 @@ int     zone;                           /* Zone number               */
 
     if(zone >= FEATURE_SIE_MAXZONES)
     {
+        PTT(PTT_CL_ERR,"*TPZI", regs->GR_L(1),(U32)(effective_addr2 & 0xffffffff),regs->psw.IA_L);
         regs->psw.cc = 0;
         return;
     }
@@ -1340,6 +1380,7 @@ U32    newgr1;
         || (dev->pmcw.flag5 & PMCW5_V) == 0
         || (dev->pmcw.flag5 & PMCW5_E) == 0)
     {
+        PTT(PTT_CL_ERR,"*DIAG002", regs->GR_L(r1),regs->GR_L(r3),regs->GR_L(1));
         regs->psw.cc = 3;
         return;
     }
